/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

//@(#) $Id: NodeColor.java 2089 2007-11-28 13:56:13Z s3460 $
package at.spardat.xma.mdl.tree;

/**
 * Used to define the color of a TreeNode.
 *  
 * @author YSD, 02.12.2004  
 */
public class NodeColor implements Cloneable {
    
    private int         fRed;
    private int         fGreen;
    private int         fBlue;

    /**
     * Constructor
     * 
     * @param red  intensity of red color (0..255)
     * @param green  intensity of green color (0..255)
     * @param blue   intensity of blue color (0..255)
     */
    public NodeColor (int red, int green, int blue) {
        if (red < 0 || red > 255) throw new IllegalArgumentException();
        if (green < 0 || green > 255) throw new IllegalArgumentException();
        if (blue < 0 || blue > 255) throw new IllegalArgumentException();
        fRed = red;
        fGreen = green;
        fBlue = blue;
    }
    
    /**
     * Constructor that sets this from a string encoded value
     */
    NodeColor (String encodedValue) {
        int     val = Integer.parseInt (encodedValue);
        fBlue = val % 256;
        val /= 256;
        fGreen = val % 256;
        val /= 256;
        fRed = val;
    }

    /**
     * @return Returns the blue value.
     */
    public int getBlue() {
        return fBlue;
    }
    
    /**
     * @return Returns the green value.
     */
    public int getGreen() {
        return fGreen;
    }
    
    /**
     * @return Returns the red value
     */
    public int getRed() {
        return fRed;
    }
    
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals (Object obj) {
        if (!(obj instanceof NodeColor)) throw new IllegalArgumentException();
        NodeColor           c2 = (NodeColor)obj;
        return fRed == c2.fRed && fGreen == c2.fGreen && fBlue == c2.fBlue;
    }
    
    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return fRed ^ fGreen ^ fBlue;
    }
    
    /**
     * Maps this color to an encoded String that may be set via the package local constructor. 
     */
    String encode () {
        return String.valueOf (((fRed * 256) + fGreen) * 256 + fBlue);
    }
    
    /**
     * Returns a clone of this;
     */
    public Object clone() {
        try {
            return super.clone();
        } catch (CloneNotSupportedException ex) {
            ex.printStackTrace();  // does not happen
        }
        return null;
    }

    
    
    public static void main(String[] args) {
        NodeColor           c = new NodeColor (23, 100, 255);
        NodeColor           c2 = new NodeColor (c.encode());
        System.out.println (c2);
    }
}
